/* -*- Mode: C; tab-width: 3; indent-tabs-mode: t; c-basic-offset: 3 -*- */
/*
  data_interpretor.c
  heraia - an hexadecimal file editor and analyser based on ghex
 
  (C) Copyright 2005 Olivier Delhomme
  e-mail : heraia@delhomme.org
  URL    : http://heraia.tuxfamily.org
 
  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2, or  (at your option) 
  any later version.
 
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY;  without even the implied warranty of
  MERCHANTABILITY  or  FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
 
  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA. */

#include <gtk/gtk.h>
#include <gtkhex/gtkhex.h>

#include "data_interpretor.h"
#include "decode.h"
#include "ghex_heraia_interface.h"

/* Begining of data interpretor things  */
/*
  Here we want to know where we are in the file (from the beginning)
*/
static void DI_print_offset( GtkWidget *hexwidget, 
							 Data_Window_Struct *DW )
{	
	guint pos;      /* position of the cursor in the file */
	gchar *text;    /* interpreted text to be printed     */
	GtkHex *gh;

	gh = GTK_HEX (hexwidget);

	pos = gtk_hex_get_cursor (gh);

	/* test if were are in the file that we are curently editing */
	if((pos >= 0) && (pos <= gh->document->file_size))
		{
			text = (gchar *) g_malloc0 (H_DI_MAX_ENTRY*sizeof(gchar));
			sprintf(text, "%u", pos);
			gtk_entry_set_text (GTK_ENTRY(DW->entry_offset), text);
			g_free (text);
		}
	else
		{
			text = (gchar *) g_malloc0 (H_DI_MAX_ENTRY*sizeof(gchar));
			sprintf(text, "%s", "Unknown");
			gtk_entry_set_text (GTK_ENTRY(DW->entry_offset), text);
			g_free (text);
		}
}

/*
    Here we do interpret an unsigned byte in decimal format
*/
static void interpret_unsigned_byte( GtkWidget *hexwidget, 
									 Data_Window_Struct *DW )
{	
	guint pos;      /* position of the cursor in the file */
	gchar *text;    /* interpreted text to be printed     */
	guchar c;       /* the character under the cursor     */
	GtkHex *gh;

	gh = GTK_HEX (hexwidget);

	pos = gtk_hex_get_cursor (gh);

	/* test if were are in the file that we are curently editing */
	if((pos >= 0) && (pos <= gh->document->file_size))
		{
			c = gtk_hex_get_byte(gh, pos);
			text = (gchar *) g_malloc0 (H_DI_MAX_ENTRY*sizeof(gchar));
			sprintf(text, "%u", c);
			gtk_entry_set_text (GTK_ENTRY(DW->entry_unsigned_byte), text);
			g_free (text);
		}
	else
		{
			text = (gchar *) g_malloc0 (H_DI_MAX_ENTRY*sizeof(gchar));
			sprintf(text, "%s", "Not a number");
			gtk_entry_set_text (GTK_ENTRY(DW->entry_unsigned_byte), text);
			g_free (text);
		}
}

/*
    Here we do interpret a signed byte in decimal format
*/
static void interpret_signed_byte( GtkWidget *hexwidget, 
								   Data_Window_Struct *DW )
{	
	guint pos;      /* position of the cursor in the file */
	gchar *text;    /* interpreted text to be printed     */
	gchar c;        /* the character under the cursor     */
	GtkHex *gh;

	gh = GTK_HEX (hexwidget);

	pos = gtk_hex_get_cursor (gh);

	/* test if were are in the file that we are curently editing */
	if((pos >= 0) && (pos <= gh->document->file_size))
		{
			c = gtk_hex_get_byte(gh, pos);
			text = (gchar *) g_malloc0 (H_DI_MAX_ENTRY*sizeof(gchar));
			sprintf(text, "%d", c);
			gtk_entry_set_text (GTK_ENTRY(DW->entry_signed_byte), text);
			g_free (text);
		}
	else
		{
			text = (gchar *) g_malloc0 (H_DI_MAX_ENTRY*sizeof(gchar));
			sprintf(text, "%s", "Not a number");
			gtk_entry_set_text (GTK_ENTRY(DW->entry_signed_byte), text);
			g_free (text);
		}
}


/*
    Here we do interpret a dos date (a 4 bytes data)
	We can not have any date struct type because our interprets
    might differ from any real date (when interpreted data does
    not represent a dos date).
*/
static void interpret_dos_date( GtkWidget *hexwidget, 
								   Data_Window_Struct *DW )
{	
	gchar *text; /* interpreted text to be printed                   */
	guchar *c;   /* the characters under the cursor and the next one */
	GtkHex *gh;  /* the GtkHex widget we want to communicate with    */
	gint result; /* used to test different results of function calls */

	gh = GTK_HEX (hexwidget);
	
	c = (guchar *) g_malloc0 (4*sizeof(guchar));
	
	result = ghex_memcpy(gh, gtk_hex_get_cursor (gh), 4, c);

	if (result == TRUE )
		{
			text = (gchar *) g_malloc0 (H_DI_MAX_ENTRY*sizeof(gchar));
			result = decode_dos_date(c, text);
			if (result == TRUE)
				gtk_entry_set_text (GTK_ENTRY(DW->entry_dos_date), text);
			g_free (text);
		}
	
	if (result == FALSE )
		{
			text = (gchar *) g_malloc0 (H_DI_MAX_ENTRY*sizeof(gchar));
			sprintf(text, "%s", "Not a dos date");
			gtk_entry_set_text (GTK_ENTRY(DW->entry_dos_date), text);
			g_free (text);
		}

	g_free (c);
}

/* End of data interpretor things  */


/* Begining of Widget related stuff  */

static gboolean delete_dt_window_event( GtkWidget *widget, 
						      GdkEvent  *event,
						      gpointer   data )
{
  gtk_widget_hide (widget);
  return TRUE;
}

static void destroy_dt_window( GtkWidget *widget,
					     gpointer   data )
{
  gtk_widget_hide (widget);
}

/*
  function called when the cursor changes its position
*/
static void refresh_data_window( GtkWidget *hexwidget, Data_Window_Struct *DW )
{
	DI_print_offset(hexwidget, DW);
	interpret_unsigned_byte(hexwidget, DW);
	interpret_signed_byte(hexwidget, DW);
	interpret_dos_date(hexwidget, DW);

	gtk_widget_show_all (DW->window);
}


/* End of Widget related stuff  */


/*
  Initialise the Data_Window_Struct whose aim is to 
  display the data interpreted. Following interpret
  is done automatically :
  - None !
*/
void data_interpret( GtkWidget *hexwidget, Data_Window_Struct *DW )
{

	DW->window = gtk_window_new(GTK_WINDOW_TOPLEVEL);
	  
	DW->window_vbox = gtk_vbox_new (FALSE, H_DI_V_SPACE);
	gtk_container_add (GTK_CONTAINER(DW->window), DW->window_vbox);
  	
	DW->window_table = gtk_table_new (H_DI_LINES, H_DI_COLUMNS, TRUE);
	gtk_container_add (GTK_CONTAINER(DW->window_vbox), DW->window_table);

	/* Bon c'est pas propre, faudra revoir a !! */

	/* offset */
	DW->label_offset = gtk_label_new ("offset");
	gtk_table_attach(GTK_TABLE(DW->window_table), GTK_WIDGET(DW->label_offset), 
					 0, 1, 0, 1, GTK_EXPAND | GTK_FILL, GTK_EXPAND, 
					 H_DI_H_SPACE, H_DI_V_SPACE);
	DW->entry_offset = gtk_entry_new_with_max_length (H_DI_MAX_ENTRY);
	gtk_table_attach(GTK_TABLE(DW->window_table), GTK_WIDGET(DW->entry_offset), 
					 1, 2, 0, 1, GTK_EXPAND | GTK_FILL, GTK_EXPAND, 
					 H_DI_H_SPACE, H_DI_V_SPACE);
	DI_print_offset(hexwidget, DW);


	/* unsigned byte */
	DW->label_unsigned_byte = gtk_label_new ("8 bits unsigned");
	gtk_table_attach(GTK_TABLE(DW->window_table), GTK_WIDGET(DW->label_unsigned_byte), 
					 0, 1, 1, 2, GTK_EXPAND | GTK_FILL, GTK_EXPAND, 
					 H_DI_H_SPACE, H_DI_V_SPACE); 
	DW->entry_unsigned_byte = gtk_entry_new_with_max_length (H_DI_MAX_ENTRY);
	gtk_table_attach(GTK_TABLE(DW->window_table), GTK_WIDGET(DW->entry_unsigned_byte), 
					 1, 2, 1, 2, GTK_EXPAND | GTK_FILL, GTK_EXPAND, 
					 H_DI_H_SPACE, H_DI_V_SPACE);
	interpret_unsigned_byte(hexwidget, DW);
 

	/* signed byte */
	DW->label_signed_byte = gtk_label_new ("8 bits signed");
	gtk_table_attach(GTK_TABLE(DW->window_table), GTK_WIDGET(DW->label_signed_byte), 
					 0, 1, 2, 3, GTK_EXPAND | GTK_FILL, GTK_EXPAND, 
					 H_DI_H_SPACE, H_DI_V_SPACE);
	DW->entry_signed_byte = gtk_entry_new_with_max_length (H_DI_MAX_ENTRY);
	gtk_table_attach(GTK_TABLE(DW->window_table), GTK_WIDGET(DW->entry_signed_byte), 
					 1, 2, 2, 3, GTK_EXPAND | GTK_FILL, GTK_EXPAND, 
					 H_DI_H_SPACE, H_DI_V_SPACE);
	interpret_signed_byte(hexwidget, DW);


	/* dos date */
	DW->label_dos_date = gtk_label_new ("dos date");
	gtk_table_attach(GTK_TABLE(DW->window_table), GTK_WIDGET(DW->label_dos_date), 
					 0, 1, 3, 4, GTK_EXPAND | GTK_FILL, GTK_EXPAND, 
					 H_DI_H_SPACE, H_DI_V_SPACE);
	DW->entry_dos_date = gtk_entry_new_with_max_length (H_DI_MAX_ENTRY);
	gtk_table_attach(GTK_TABLE(DW->window_table), GTK_WIDGET(DW->entry_dos_date), 
					 1, 2, 3, 4, GTK_EXPAND | GTK_FILL, GTK_EXPAND, 
					 H_DI_H_SPACE, H_DI_V_SPACE);
	interpret_dos_date(hexwidget, DW);


	g_signal_connect (G_OBJECT (DW->window), "delete_event", 
					  G_CALLBACK (delete_dt_window_event), NULL);
	g_signal_connect (G_OBJECT (DW->window), "destroy", 
					  G_CALLBACK (destroy_dt_window), NULL);

	g_signal_connect (G_OBJECT (hexwidget), "cursor_moved",
					  G_CALLBACK (refresh_data_window), DW);

	gtk_widget_show_all (DW->window);
}
